@@ -14,9 +14,13 @@
 
 
 from _warnings import warn
+from typing import Tuple
+
 import matplotlib
 from batchgenerators.utilities.file_and_folder_operations import *
+from nnunet.network_architecture.neural_network import SegmentationNetwork
 from sklearn.model_selection import KFold
+from torch import nn
 from torch.optim.lr_scheduler import _LRScheduler
 
 matplotlib.use("agg")
@@ -30,6 +34,7 @@ from collections import OrderedDict
 import torch.backends.cudnn as cudnn
 from abc import abstractmethod
 from datetime import datetime
+from tqdm import trange
 
 try:
     from apex import amp
@@ -60,7 +65,8 @@ class NetworkTrainer(object):
         if deterministic:
             np.random.seed(12345)
             torch.manual_seed(12345)
-            torch.cuda.manual_seed_all(12345)
+            if torch.cuda.is_available():
+                torch.cuda.manual_seed_all(12345)
             cudnn.deterministic = True
             torch.backends.cudnn.benchmark = False
         else:
@@ -68,7 +74,7 @@ class NetworkTrainer(object):
             torch.backends.cudnn.benchmark = True
 
         ################# SET THESE IN self.initialize() ###################################
-        self.network = None
+        self.network: Tuple[SegmentationNetwork, nn.DataParallel] = None
         self.optimizer = None
         self.lr_scheduler = None
         self.tr_gen = self.val_gen = None
@@ -113,6 +119,10 @@ class NetworkTrainer(object):
         self.log_file = None
         self.deterministic = deterministic
 
+        self.use_progress_bar = False
+        if 'nnunet_use_progress_bar' in os.environ.keys():
+            self.use_progress_bar = bool(int(os.environ['nnunet_use_progress_bar']))
+
     @abstractmethod
     def initialize(self, training=True):
         """
@@ -121,9 +131,9 @@ class NetworkTrainer(object):
         modify self.output_folder if you are doing cross-validation (one folder per fold)
 
         set self.tr_gen and self.val_gen
-        
+
         call self.initialize_network and self.initialize_optimizer_and_scheduler (important!)
-        
+
         finally set self.was_initialized to True
         :param training:
         :return:
@@ -194,7 +204,7 @@ class NetworkTrainer(object):
 
             if len(self.all_val_losses_tr_mode) > 0:
                 ax.plot(x_values, self.all_val_losses_tr_mode, color='g', ls='-', label="loss_val, train=True")
-            if len(self.all_val_eval_metrics) == len(self.all_val_losses):
+            if len(self.all_val_eval_metrics) == len(x_values):
                 ax2.plot(x_values, self.all_val_eval_metrics, color='g', ls='--', label="evaluation metric")
 
             ax.set_xlabel("epoch")
@@ -375,7 +385,7 @@ class NetworkTrainer(object):
 
     def _maybe_init_amp(self):
         # we use fp16 for training only, not inference
-        if self.fp16:
+        if self.fp16 and torch.cuda.is_available():
             if not self.amp_initialized:
                 if amp is not None:
                     self.network, self.optimizer = amp.initialize(self.network, self.optimizer, opt_level="O1")
@@ -396,7 +406,8 @@ class NetworkTrainer(object):
         _ = self.tr_gen.next()
         _ = self.val_gen.next()
 
-        torch.cuda.empty_cache()
+        if torch.cuda.is_available():
+            torch.cuda.empty_cache()
 
         self._maybe_init_amp()
 
@@ -419,9 +430,20 @@ class NetworkTrainer(object):
 
             # train one epoch
             self.network.train()
-            for b in range(self.num_batches_per_epoch):
-                l = self.run_iteration(self.tr_gen, True)
-                train_losses_epoch.append(l)
+
+            if self.use_progress_bar:
+                with trange(self.num_batches_per_epoch) as tbar:
+                    for b in tbar:
+                        tbar.set_description("Epoch {}/{}".format(self.epoch+1, self.max_num_epochs))
+
+                        l = self.run_iteration(self.tr_gen, True)
+
+                        tbar.set_postfix(loss=l)
+                        train_losses_epoch.append(l)
+            else:
+                for _ in range(self.num_batches_per_epoch):
+                    l = self.run_iteration(self.tr_gen, True)
+                    train_losses_epoch.append(l)
 
             self.all_tr_losses.append(np.mean(train_losses_epoch))
             self.print_to_log_file("train loss : %.4f" % self.all_tr_losses[-1])
@@ -505,8 +527,8 @@ class NetworkTrainer(object):
         else:
             if len(self.all_val_eval_metrics) == 0:
                 """
-                We here use alpha * old - (1 - alpha) * new because new in this case is the vlaidation loss and lower 
-                is better, so we need to negate it. 
+                We here use alpha * old - (1 - alpha) * new because new in this case is the vlaidation loss and lower
+                is better, so we need to negate it.
                 """
                 self.val_eval_criterion_MA = self.val_eval_criterion_alpha * self.val_eval_criterion_MA - (
                         1 - self.val_eval_criterion_alpha) * \
@@ -600,8 +622,9 @@ class NetworkTrainer(object):
         if not isinstance(target, torch.Tensor):
             target = torch.from_numpy(target).float()
 
-        data = data.cuda(non_blocking=True)
-        target = target.cuda(non_blocking=True)
+        if torch.cuda.is_available():
+            data = data.cuda(non_blocking=True)
+            target = target.cuda(non_blocking=True)
 
         self.optimizer.zero_grad()
         output = self.network(data)
@@ -614,7 +637,7 @@ class NetworkTrainer(object):
         del target
 
         if do_backprop:
-            if not self.fp16 or amp is None:
+            if not self.fp16 or amp is None or not torch.cuda.is_available():
                 l.backward()
             else:
                 with amp.scale_loss(l, self.optimizer) as scaled_loss:
